/*-----------------------------------------------------------------------*/
/*  (C) Copyright Christian Monstein HB9SCT 8807 Freienbach              */
/*-----------------------------------------------------------------------*/
/*  Programmname: fixstar.CPP                                            */
/*                                                                       */
/*  Ausgabe: V2.1     Datum: 02.05.1998    Autor: Chr. Monstein          */
/*                                                                       */
/*  Zweck: Berechnet nach MEEUS alle Koordinaten fr fixstar-source      */
/*                                                                       */
/*  Source: fixstar.CPP    Compiler: C++5.01                             */
/*                                                                       */
/*-----------------------------------------------------------------------*/

// Created by: Jeffrey Sax   01.11.1992   MEEUS...
// Updated by: Chr. Monstein 06.04.1997   reduziert fr MUON-Telescope
// Updated by: Chr. Monstein 02.05.1998   converted to C++

#include <stdio.h>
#include <math.h>

#include "fixstar.h"
#include "aa.h"

//-------------------------------------------------------------------


void FixstarCalc(double Longitude, double Latitude, double TZ, double H0,
               int year, int month, int day, double hh,
               double ra, double dec,
               double *lha, double *azi, double *ele, double *JulianDay)
{
  TDATE Date;
  double ObsLon, ObsLat, Height;
  double T, DTOfUT, DeltaT, LAST;
  double Obl, NutLon, NutObl, TimeZone;
  double RA, Decl;
  double RA2, Decl2;
  double Az, Alt, dAlt;
  double tempJD;

  ObsLon = Longitude*DToR;
  ObsLat = Latitude*DToR;
  TimeZone = TZ/24.0;
  Height = H0;
  Date.yy=year;  // Jahr
  Date.mm=month; // Monat
  Date.dd=day;   // Ganzer Tag
  Date.dd=Date.dd+hh/24.0; // plus angebrochener Tag (Stunden)

  // vorlaeufige Berechnungen
  tempJD = CalendarToJD (&Date);
  *JulianDay = tempJD;
  T = JDToT (tempJD + TimeZone);
  DeltaT = ApproxDeltaT (T);
  DTOfUT = T + DeltaT * secToT;
  InitPlanetaryTerms(0);
  Obl = Obliquity(T);
  NutationConst (T, &NutLon, &NutObl);
  LAST = SiderealTime (DTOfUT) + NutLon * cos (Obl) - ObsLon;
  while (LAST>pi2) LAST=LAST-pi2;
  *lha=LAST; // Rckgabewert local sideral time

  RA=ra;
  Decl=dec;
  EquToHor (RA, Decl, LAST, ObsLat, &Az, &Alt);
  dAlt=0; // keine Refraktion aus Prinzip
  if (Alt > 0) // murks, weil (Alt> -h0Planet)!
  {
    dAlt = AtmRefraction (Alt);
  }
  *azi=Az*RToD+180.0; // Azimut
  *ele=(Alt+dAlt)*RToD; // Elevation
  FreeAllPlanetaryTerms ();
}
//-----------------------------------------------------------------