/*-----------------------------------------------------------------------*/
/*  (C) Copyright Christian Monstein HB9SCT 8807 Freienbach              */
/*-----------------------------------------------------------------------*/
/*  Programmname: Moon.CPP                                               */
/*                                                                       */
/*  Ausgabe: V2.1     Datum: 01.05.1998    Autor: Chr. Monstein          */
/*                                                                       */
/*  Zweck: Berechnet nach MEEUS alles fuer den Mond                      */
/*                                                                       */
/*  Source: moon.CPP    Compiler: C++5.01                                */
/*                                                                       */
/*-----------------------------------------------------------------------*/

// Created by: Jeffrey Sax   01.11.1992   MEEUS...
// Updated by: Chr. Monstein 06.04.1997   reduziert fr MUON-Telescope
// Updated by: Chr. Monstein 02.05.1998   converted to C++

#include <stdio.h>
#include <math.h>

#include "Moon.h"
#include "aa.h"

void MoonCalc(double Longitude, double Latitude, double TZ, double H0,
               int year, int month, int day, double hh,
               double *lha, double *azi, double *ele,
               double *rightascension, double *declination, double *JulianDay)
{

  TDATE Date;
  double ObsLon, ObsLat, Height, TimeZone;
  double RhoCosPhi, RhoSinPhi;
  double T, DTOfUT, DeltaT;
  double Obl, NutLon, NutObl, LAST;
  TSVECTOR S, S2, SSun;
  double MoonParallax, SemiDiameter;
  double RA, Decl;
  double RA1, RA2, RA3;
  double Decl1, Decl2, Decl3;
  double Az, Alt, dAlt;
  double Rise, Transit, Setting;
  double tempJD;
  int RiseSetFlags;
  TMOONPHYSDATA PhysData;

  ObsLon = Longitude*DToR;
  ObsLat = Latitude*DToR;
  TimeZone = TZ/24.0;
  Height = H0;
  Date.yy=year;  // Jahr
  Date.mm=month; // Monat
  Date.dd=day;   // Ganzer Tag
  Date.dd=Date.dd+hh/24.0; // plus angebrochener Tag (Stunden)

  //printf("Position des Mondes am ");
  //WriteDate(&Date);
  //printf(" um ");
  //WriteHMS((Date.dd - floor(Date.dd)) * pi2, 4);
  //printf("\nLaenge = ");
  //WriteDMS(ObsLon, 4);
  //printf(", Breite = ");
  //WriteDMS(ObsLat, 4);
  //printf(", Hoehe ueber dem Meeresspiegel : %.0fm\n\n", Height);

  // Vorlaeufige Berechnung
  tempJD = CalendarToJD (&Date);
  *JulianDay = tempJD;
  T = JDToT (tempJD + TimeZone);
  DeltaT = ApproxDeltaT (T);
  DTOfUT = T + DeltaT * secToT;
  Obl = Obliquity(T);
  NutationConst (T, &NutLon, &NutObl);
  //LAST = SiderealTime (T) + NutLon * cos(Obl) - ObsLon; // Originalvariante Meeus(Mond) vermutlich falsch!
  LAST = SiderealTime (DTOfUT) + NutLon * cos(Obl) - ObsLon; // Variante (Kopie) aus Planetenrechnerei
  while (LAST>pi2) LAST=LAST-pi2;
  *lha=LAST; // Rckgabewert local sideral time

  ObserverCoord (ObsLat, Height, &RhoCosPhi, &RhoSinPhi);

  // Berechnet die Position der Sonne
  InitPlanetaryTerms (0); // bisher ("")
  ReadPlanetaryTerms (3);
  SunPosHi (T, EPDATE, &SSun);
  FreePlanetaryTerms (3);

  // Hauptberechnung
  MoonPos (T, &S);
  MoonParallax = asin (EarthRadius / S.r);
  SemiDiameter = MoonSemiDiameter (S.r);
  //printf("Scheinbare ekliptikale Koordinaten : lambda = ");
  //WriteDMS (S.l + NutLon, 5);
  //printf (", Beta = ");
  //WriteDMS (S.b, 4);
  //putchar ('\n');

  //printf ("Parallaxe = ");
  //WriteDMS (MoonParallax, 7);
  //printf (", geozentrischer Winkeldurchmesser = ");
  //WriteDMS (2 * SemiDiameter * SToR, 4);
  //putchar ('\n');
  //printf ("Abstand Erde-Mond = %8.1fkm = %8.1fmi\n", S.r, S.r/1.60934);

  EclToEqu (S.l, S.b, Obl, &RA, &Decl);
  //printf ("mittlere aequatoriale Koordinaten : alpha = ");
  //WriteHMS (RA, 5);
  //printf (", delta = ");
  //WriteDMS (Decl, 4);
  //putchar ('\n');

  EclToEqu (S.l + NutLon, S.b, Obl + NutObl, &RA, &Decl);
  //printf ("scheinbare aequatoriale Koordinaten : alpha = ");
  //WriteHMS (RA, 5);
  //printf (", delta = ");
  //WriteDMS (Decl,4);
  //putchar ('\n');

  ParallaxHiAlt (sin(MoonParallax), LAST, RhoCosPhi, RhoSinPhi, &RA, &Decl, &SemiDiameter);
  //printf ("topozentrische Koordinaten : alpha = ");
  //WriteHMS (RA, 5);
  //printf (", delta = ");
  //WriteDMS (Decl, 4);
  //putchar ('\n');
  //printf ("topozentrischer Winkeldurchmesser = ");
  //WriteDMS (2 * SemiDiameter * SToR, 4);
  //putchar ('\n');
  *rightascension=RA;
  *declination=Decl;

  EquToHor (RA, Decl, LAST, ObsLat, &Az, &Alt);
  //printf ("Horizontkoordinaten : A = ");
  //WriteDMS (Az, 4);
  //printf (", h = ");
  //WriteDMS (Alt, 4);
  //putchar ('\n');
  dAlt=0; // per Definition...
  if (Alt > 0)
  {
    dAlt = AtmRefraction (Alt);
    //printf ("Hoehe unter Einbeziehung der atmosphaerischen Refraktion : ");
    //WriteDMS (Alt + dAlt, 4);
    //putchar ('\n');
  }
  *azi=Az*RToD+180.0; // Azimut
  *ele=(Alt+dAlt)*RToD; // Elevation
  //putchar ('\n');

  // Physische Ephemeride
  //puts ("Physische Ephemeride :");
  MoonPhysEphemeris(T, &S, &SSun, Obl, NutLon, NutObl, &PhysData);
  //printf ("  Phasenwinkel = %4.2f, phase = %5.3f\n",
  //  PhysData.i * RToD, PhysData.k);
  //printf ("  Positionswinkel des hellen Mondrandes = %3.1f\n",
  //  PhysData.X * RToD);
  //printf ("  optische Libration :l' = %6.3f, b' = %6.3f\n",
  //  PhysData .ld * RToD, PhysData.bd * RToD);
  //printf ("  physische Libration :l\" = %6.3f, b\" = %6.3f\n",
  //  PhysData.ldd * RToD, PhysData.bdd * RToD);
  //printf ("  Gesamtlibration :l  = %6.3f, b  = %6.3f\n",
  //  PhysData.l * RToD, PhysData.b * RToD);
  //printf ("  Positionswinkel : %4.2f", PhysData.P * RToD);
  //putchar ('\n');

  // Berechnet die Zeiten von Mondaufgang und -untergang
  T = (floor (T * 36525 - 0.5 - TimeZone) + 0.5 + TimeZone) / 36525;
  DTOfUT = T + DeltaT*secToT;
  MoonPos (DTOfUT - 1 / 36525.0, &S2);
  EclToEqu (S2.l, S2.b, Obl, &RA1, &Decl1);
  MoonPos (DTOfUT, &S2);
  EclToEqu (S2.l, S2.b, Obl, &RA2, &Decl2);
  MoonPos (DTOfUT + 1 / 36525.0, &S2);
  EclToEqu (S2.l, S2.b, Obl, &RA3, &Decl3);
  RiseSet (T, DeltaT,
    RA1, Decl1, RA2, Decl2, RA3, Decl3,
    MoonSetHeight (MoonParallax), ObsLon, ObsLat,
    &Rise, &Transit, &Setting, &RiseSetFlags);
  switch (RiseSetFlags)
  {
    case ALWAYS_ABOVE:
      //printf("Der Mond ist immer ueber dem Horizont.  Meridiandurchgang um ");
      //WriteHMS(Transit, 2);
      //printf(".\n");
      break;
    case ALWAYS_BELOW:
      //puts("Der Mond ist immer unter dem Horizont.");
      break;
    default:
      //printf ("Mondaufgang : ");
      //WriteHMS(Rise,2);
      //printf (", Durchgang : ");
      //WriteHMS(Transit,2);
      //printf (", Monduntergang : ");
      //WriteHMS(Setting, 2);
      ;
  }
  //putchar ('\n');

}