//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdio.h>
#include <fstream.h>
#include <printers.hpp>

#include "aa.h"
#include "helio.h"
#include "moon.h"
#include "fixstar.h"
#include "Sternzei.h"
#include "About.h"
#include "Help.h"
#include "Satellit.h"
#include "Unit2.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
#define NumberOfObservers 9
TForm1 *Form1;
Word zeit=0;
int timezone;
enum sourcetyp {solar, moon, fixstar} sourcetypen;
int planet; // 0=sun, ...3=earth
int dt=30; // initial timestep [minutes]

struct ObserversParameter
{
    char Name[40];
    double Longitude;
    double Latitude;
    double Heigth;
    int Timezone;
} Observer[NumberOfObservers];

double heigth;
double laenge;
double breite;
double rightascension, declination;
double JulianDay;
char sourcename[40];
char observername[40];
float HorizonLevel[361];

extern double GMST;
bool first=true;
bool table=false;
bool ViewHorizon=true;
bool GeoCurve=true;


//---------------------------------------------------------------------------

void __fastcall TForm1::ReadObservers(void)
{
    char buff[80];
    int i=0;
    // read cfg-file
    ifstream infile("observer.cfg");
    for (i=0; i<NumberOfObservers; i++)
    {
      infile.getline(Observer[i].Name,40);
      infile.getline(buff,40); Observer[i].Heigth    = (double)atof(buff);
      infile.getline(buff,40); Observer[i].Longitude = (double)atof(buff);
      infile.getline(buff,40); Observer[i].Latitude  = (double)atof(buff);
      infile.getline(buff,40); Observer[i].Timezone  =         atoi(buff);
    }
    infile.close();
    HB9SCT1   ->Caption=Observer[0].Name;
    DC1GS1    ->Caption=Observer[1].Name;
    DJ2US1    ->Caption=Observer[2].Name;
    HB9CKL    ->Caption=Observer[3].Name;
    HB9XM1    ->Caption=Observer[4].Name;
    PHOENIX21 ->Caption=Observer[5].Name;
    PRAXOS1   ->Caption=Observer[6].Name;
    Greenwich1->Caption=Observer[7].Name;
    Unknown   ->Caption=Observer[8].Name;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ReadHorizon(void)
{
    char buff[80];
    int i=0;
    ifstream infile("horizon.prn");
    for (i=0; i<360; i++)
    {
      infile.getline(buff,40);
      HorizonLevel[i] = atof(buff);
    }
    infile.close();
}
//---------------------------------------------------------------------------
void __fastcall TForm1::ReadLocalPar(void)
{
    char buff[40];
    ifstream infile("astro.cfg");
    infile.getline(observername,40);
    infile.getline(buff,40); heigth         = (double)atof(buff);
    infile.getline(buff,40); laenge         = (double)atof(buff);
    infile.getline(buff,40); breite         = (double)atof(buff);
    infile.getline(buff,40); timezone       =         atoi(buff);

    infile.getline(buff,40); sourcetypen    = (sourcetyp)atoi(buff);
    infile.getline(sourcename,40);
    infile.getline(buff,40); rightascension = (double)atof(buff);
    infile.getline(buff,40); declination    = (double)atof(buff);
    infile.getline(buff,40); planet         =         atoi(buff);
    infile.close();
}
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
    : TForm(Owner)
{
    ReadObservers(); // name, coordinates
    ReadHorizon();

    Form1->Width=ListBox1->Left + ListBox1->Width + 20;
    Form1->Height=ListBox1->Top + ListBox1->Height + 40;

    // read cfg-file
    ReadLocalPar();
    
    Timer1->Interval=10;
    Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void DezimalToSexagesimal(double temp, Word *Hour, Word *Min, Word *Sec)
{
    Word h,m,s;
    h=(Word)temp;
    m=(Word)(60*(temp-h));
    s=(Word)(60*(60*(temp-h)-m)+0.5);
    *Hour=h;
    *Min =m;
    *Sec =s;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Timer1Timer(TObject *Sender)
{
    double lha=0, azi=0, ele=0;
    char st[80];
    Word Year, Month, Day, Hour, Min, Sec, MSec;
    double h_dec, temp;
    static Word act=-1, alt=0;

    if (zeit==10)
    {
        zeit=0;
        TDateTime dtPresent = Now();
        DecodeDate(dtPresent, Year, Month, Day);
        sprintf(st,"Date: %02u-%02u-%04u",Day,Month,Year);
        Label1->Caption = st;
        DecodeTime(dtPresent, Hour, Min, Sec, MSec);
        act=Sec;
      //sprintf(st,"Time (PC):  %02u:%02u:%02u.%02u",Hour,Min,Sec,MSec/10);
      //sprintf(st,"Local time: %02u:%02u:%02u.%01u",Hour,Min,Sec,MSec/100);
        sprintf(st,"Local time: %02u:%02u:%02u",Hour,Min,Sec);
        Label2->Caption = st;

        h_dec=(double)Hour+(double)Min/60.0+(double)Sec/3600.0;
        switch (sourcetypen)
        {
            case solar:
            {
                AstroCalc(laenge, breite, timezone, heigth, planet,
                  Year, Month, Day, h_dec,
                  &lha, &azi, &ele, &rightascension, &declination,
                  &JulianDay);
                break;
            }
            case moon:
            {
                MoonCalc(laenge, breite, timezone, heigth,
                  Year, Month, Day, h_dec,
                  &lha, &azi, &ele, &rightascension, &declination,
                  &JulianDay);
                break;
            }
            case fixstar:
            {
                FixstarCalc(laenge, breite, timezone, heigth,
                  Year, Month, Day, h_dec, rightascension, declination,
                  &lha, &azi, &ele,
                  &JulianDay);
                break;
            }
            default:;
        }
        DezimalToSexagesimal(lha*RToH, &Hour, &Min, &Sec);
        sprintf(st,"LST= %6.2f = %02u:%02u:%02u",lha*RToD,Hour,Min,Sec);
        Label3->Caption = st;

        temp=lha*RToH+laenge*DToR*RToH; // Dezimalstunde
        while (temp<  0) temp=temp+24;
        while (temp>=24) temp=temp-24;
        DezimalToSexagesimal(temp, &Hour, &Min, &Sec);
        sprintf(st,"GHA= %6.2f = %02u:%02u:%02u",temp*HToR*RToD,Hour,Min,Sec);
        Label5->Caption = st;

        sprintf(st,"AZI=%7.2f; ELE=%6.2f",azi,ele);
        Label4->Caption = st;

        DezimalToSexagesimal(rightascension*RToH, &Hour, &Min, &Sec);
        sprintf(st,"RA =%02u:%02u:%02u; DEC=%6.2f",Hour,Min,Sec,declination*RToD);
        Label6->Caption = st;

        sprintf(st,"JD =%15.5f",JulianDay);
        Label7->Caption = st;

        temp = JulianDay-2440000.5;
        sprintf(st,"TJD=%6i D=%6.0fs",int(temp),86400.0*(temp-int(temp)));
        Label8->Caption = st;

        switch (timezone)
        {
            case  0: Button1->Caption=AnsiString("Timezone: UT");      break;
            case -1: Button1->Caption=AnsiString("Timezone: MEZ");     break;
            case -2: Button1->Caption=AnsiString("Timezone: MESZ");    break;
            default: Button1->Caption=AnsiString("Timezone:unknown");  break;
        }
        Button2->Caption=AnsiString("Observatory: ") + observername;
        Button3->Caption=AnsiString("Sourceobject: ")+ sourcename;

        if (act != alt) // second changed?
        {
            alt=act;
            if (first)
            {
                Button5Click(0); // full curve
                first=false;
            }
            else
            {
                DrawCurve(0); // only actual value
                //if (GeoCurve) DrawGeostatcurve(0);
            }

        }
    }
    zeit++;
}
//---------------------------------------------------------------------------
void __fastcall TForm1::MESZClick(TObject *Sender)
{
    timezone=-2;
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::UTClick(TObject *Sender)
{
    timezone=0;
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MEZClick(TObject *Sender)
{
    timezone=-1;
    first=true;
}
//---------------------------------------------------------------------------


void __fastcall TForm1::Button1Click(TObject *Sender)
{
    TimeZone->Popup(Button1->Left+Form1->Left+Button1->Width,
                    Button1->Top +Form1->Top);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button2Click(TObject *Sender)
{
    Koordinaten->Popup(Button2->Left+Form1->Left+Button2->Width,
                       Button2->Top +Form1->Top);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button3Click(TObject *Sender)
{
    Object->Popup(Button3->Left+Form1->Left+Button3->Width,
                  Button3->Top +Form1->Top);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SunClick(TObject *Sender)
{
    planet=0;
    sourcetypen=solar;
    sprintf(sourcename,"Sun");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MerkurClick(TObject *Sender)
{
    planet=1;
    sourcetypen=solar;
    sprintf(sourcename,"Mercury");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::VenusClick(TObject *Sender)
{
    planet=2;
    sourcetypen=solar;
    sprintf(sourcename,"Venus");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::EarthClick(TObject *Sender)
{
    planet=3;
    sourcetypen=solar;
    sprintf(sourcename,"Earth");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MarsClick(TObject *Sender)
{
    planet=4;
    sourcetypen=solar;
    sprintf(sourcename,"Mars");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::JupiterClick(TObject *Sender)
{
    planet=5;
    sourcetypen=solar;
    sprintf(sourcename,"Jupiter");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SaturnClick(TObject *Sender)
{
    planet=6;
    sourcetypen=solar;
    sprintf(sourcename,"Saturn");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::UranusClick(TObject *Sender)
{
    planet=7;
    sourcetypen=solar;
    sprintf(sourcename,"Uranus");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::NeptunClick(TObject *Sender)
{
    planet=8;
    sourcetypen=solar;
    sprintf(sourcename,"Neptun");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::PlutoClick(TObject *Sender)
{
    planet=9;
    sourcetypen=solar;
    sprintf(sourcename,"Pluto");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MoonClick(TObject *Sender)
{
    planet=3;
    sourcetypen=moon;
    sprintf(sourcename,"Moon");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::HB9SCT1Click(TObject *Sender)
{
    // Chr. Monstein Freienbach, HB9SCT
    heigth=Observer[0].Heigth;
    laenge=Observer[0].Longitude;
    breite=Observer[0].Latitude;
    timezone=Observer[0].Timezone;
    sprintf(observername,Observer[0].Name);
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::DC1GS1Click(TObject *Sender)
{
    // Bhl/Balzhofen Klaus Steger, DC1GS
    heigth=Observer[1].Heigth;
    laenge=Observer[1].Longitude;
    breite=Observer[1].Latitude;
    timezone=Observer[1].Timezone;
    sprintf(observername,Observer[1].Name);
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::DJ2US1Click(TObject *Sender)
{
    // Helmut Oser Achern, DJ2US
    heigth=Observer[2].Heigth;
    laenge=Observer[2].Longitude;
    breite=Observer[2].Latitude;
    timezone=Observer[2].Timezone;
    sprintf(observername,Observer[2].Name);
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::HB9CKLClick(TObject *Sender)
{
    // Ricken Sd Leon Klin, HB9CKL
    heigth=Observer[3].Heigth;
    laenge=Observer[3].Longitude;
    breite=Observer[3].Latitude;
    timezone=Observer[3].Timezone;
    sprintf(observername,Observer[3].Name);
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::HB9XM1Click(TObject *Sender)
{
    // P. Aubry Ottoberg, HB9XM
    heigth=Observer[4].Heigth;
    laenge=Observer[4].Longitude;
    breite=Observer[4].Latitude;
    timezone=Observer[4].Timezone;
    sprintf(observername,Observer[4].Name);
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::PHOENIX21Click(TObject *Sender)
{
    // Bleien Grnichen, Phoenix-2
    heigth=Observer[5].Heigth;
    laenge=Observer[5].Longitude;
    breite=Observer[5].Latitude;
    timezone=Observer[5].Timezone;
    sprintf(observername,Observer[5].Name);
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::PRAXOS1Click(TObject *Sender)
{
    // Sternwarte Zrich Praktikumsempfnger Praxos
    heigth=Observer[6].Heigth;
    laenge=Observer[6].Longitude;
    breite=Observer[6].Latitude;
    timezone=Observer[6].Timezone;
    sprintf(observername,Observer[6].Name);
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Greenwich1Click(TObject *Sender)
{
    // Greenwich observatory
    heigth=Observer[7].Heigth;
    laenge=Observer[7].Longitude;
    breite=Observer[7].Latitude;
    timezone=Observer[7].Timezone;
    sprintf(observername,Observer[7].Name);
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::UnknownClick(TObject *Sender)
{
    // anybody observatory
    heigth=Observer[8].Heigth;
    laenge=Observer[8].Longitude;
    breite=Observer[8].Latitude;
    timezone=Observer[8].Timezone;
    sprintf(observername,Observer[8].Name);
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CassA1Click(TObject *Sender)
{
    // AR1950: ra=23:21:07; d=58 33.8'
    // AR2000: 350,8500000 +58,8150000 SIMBAD
    // 11000fu
    sourcetypen=fixstar;
    rightascension = 350.8500000*DToR;
    declination    =  58.8150000*DToR;
    sprintf(sourcename,"Cass A");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CygnusA1Click(TObject *Sender)
{
    // AR1950: ra=19:57:45; d=40 36'
    // AR2000: 299.8681667 +40.7338611 SIMBAD
    // 8100fu
    sourcetypen=fixstar;
    rightascension = 299.8681667*DToR;
    declination    =  40.7338611*DToR;
    sprintf(sourcename,"Cygn A");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::TaurusA1Click(TObject *Sender)
{
    // AR1950: ra=05:31:30; d=21 58'
    // AR2000: 83.6328631 +22.0143489 SIMBAD
    // 1420fu
    sourcetypen=fixstar;
    rightascension = 83.6328631*DToR;
    declination    = 22.0143489*DToR;
    sprintf(sourcename,"Tau A");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::VirgoA1Click(TObject *Sender)
{
    // AR1950: ra=12:28:18; d=12 40'
    // AR2000: 187.7059306  +12.3911233 SIMBAD
    // 970fu
    sourcetypen=fixstar;
    rightascension = 187.7059306*DToR;
    declination    =  12.3911233*DToR;
    sprintf(sourcename,"Virgo A");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SagittariusA1Click(TObject *Sender)
{
    // AR1950: ra=17:42.9; d=-28 50'
    // AR2000: 266.3012914  -28.8048772 SIMBAD
    // 1800fu
    sourcetypen=fixstar;
    rightascension = 266.3012914*DToR;
    declination    = -28.8048772*DToR;
    sprintf(sourcename,"Sagitt A");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::OrionM421Click(TObject *Sender)
{
    // AR1950: ra=05:32:49; d=-05 25'
    // AR2000: 83.8220833  -5.3911111 SIMBAD
    // 360fu
    sourcetypen=fixstar;
    rightascension = 83.8220833*DToR;
    declination    = -5.3911111*DToR;
    sprintf(sourcename,"Orion");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FixstarClick(TObject *Sender)
{
    // AR1950: ra=05:32:49; d=-05 25'
    // AR2000: 83.8220833  -5.3911111 SIMBAD
    // 360fu
    // manuelle Eingabe der Koordinaten hier
    char st[40];
    sprintf(st,"%6.3f",rightascension/DToR); Manual->Edit1->Text=st;
    sprintf(st,"%6.3f",declination   /DToR); Manual->Edit2->Text=st;
    Manual->Edit3->Text=sourcename;
    Manual->ShowModal();
    sourcetypen=fixstar;
    //rightascension = 180.0*DToR;
    //declination    = 0.0*DToR;
    //sprintf(sourcename,"Fixstar");
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::AboutButtonClick(TObject *Sender)
{
    AboutBox->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::BitBtn1Click(TObject *Sender)
{
    Application->Terminate();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::BitBtn2Click(TObject *Sender)
{
    HelpForm->Show();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button4Click(TObject *Sender)
{
    // Save actual Parameter
    ofstream outfile("astro.cfg");
    if (outfile)
    {
        outfile << observername           << endl;
        outfile << (double)heigth         << endl;
        outfile << (double)laenge         << endl;
        outfile << (double)breite         << endl;
        outfile << timezone               << endl;

        outfile << sourcetypen            << endl;
        outfile << sourcename             << endl;
        outfile << (double)rightascension << endl;
        outfile << (double)declination    << endl;
        outfile << planet                 << endl;

        outfile.close();
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::DrawCurve(TObject *Sender) // dynamic part
{
    double lha=0, azi=0, ele=0;
    Word Year, Month, Day, Hour, Min, Sec, MSec;
    double h_dec;
    int i, px, py;
    int N, N1;
    N=24*60/dt;
    N1=N+1;
    char st[80];

    TDateTime dtPresent = Now();
    DecodeDate(dtPresent, Year, Month, Day);
    DecodeTime(dtPresent, Hour, Min, Sec, MSec);

    if (table || first) ListBox1->Clear(); // 8-ung waechst!

    for (i=0; i<=N1; i++)
    {
        if (i==N1) // actual value
            h_dec=(double)Hour+(double)Min/60.0+(double)Sec/3600.0;
        else  // f(time)
            h_dec=(double)i/(double)N*24.0;

        switch (sourcetypen)
        {
            case solar:
            {
                AstroCalc(laenge, breite, timezone, heigth, planet,
                  Year, Month, Day, h_dec,
                  &lha, &azi, &ele, &rightascension, &declination,
                  &JulianDay);
                break;
            }
            case moon:
            {
                MoonCalc(laenge, breite, timezone, heigth,
                  Year, Month, Day, h_dec,
                  &lha, &azi, &ele, &rightascension, &declination,
                  &JulianDay);
                break;
            }
            case fixstar:
            {
                FixstarCalc(laenge, breite, timezone, heigth,
                  Year, Month, Day, h_dec, rightascension, declination,
                  &lha, &azi, &ele,
                  &JulianDay);
                break;
            }
            default:;
        }
        if ((i<N1) && (((int)(h_dec*60)%dt)==0)) // table without actual value
        {
            sprintf(st,"%04.1fh a=%06.2f  e=%+05.2f",h_dec,azi,ele);
            if (table || first) ListBox1->Items->Add(st);
        }

        if ((ViewHorizon) &&
            (ele>HorizonLevel[(int)azi]) ||
            ((!ViewHorizon)&& (ele>0)) )
        {
          px=(int)((double)PaintBox1->Width /360.0*azi);
          py=(int)((double)PaintBox1->Height/90.0*(90.0-ele));
          PaintBox1->Canvas->Pixels[px][py]=clLime;

          if (i==N1) // actual value
          {
            PaintBox1->Canvas->Pen->Color = clLime; // clRed
            PaintBox1->Canvas->Pen->Width = 2; // strichstaerke
            PaintBox1->Canvas->MoveTo(px,py-4);
            PaintBox1->Canvas->LineTo(px,py+4);
            PaintBox1->Canvas->MoveTo(px-4,py);
            PaintBox1->Canvas->LineTo(px+4,py);
            Label4->Font->Color=clGreen;
            //Label4->Color=clBlack; // background
          }
        } // end ele>0
        else
            Label4->Font->Color=clRed;
    }
    table =false;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button5Click(TObject *Sender) // Graph init (redraw)
{
    TRect NewRect = Rect(0, 0, PaintBox1->Width, PaintBox1->Height);

    PaintBox1->Canvas->Brush->Color = clNavy; // evtl. clBlack
    PaintBox1->Canvas->FillRect(NewRect);
    PaintBox1->Canvas->FrameRect(NewRect);
    PaintBox1->Canvas->Pen->Width = 2; // strichstaerke

    DrawCurve(0);
    if (GeoCurve) DrawGeostatcurve(0);
    if (ViewHorizon) DrawHorizon(0);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button6Click(TObject *Sender)
{
    Timeincrement->Popup(Button6->Left+Form1->Left+Button6->Width,
                         Button6->Top +Form1->Top);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::INC1h1Click(TObject *Sender)
{
    dt=60;
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::INC301Click(TObject *Sender)
{
    dt=30;
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::INC101Click(TObject *Sender)
{
    dt=12;
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::INC61Click(TObject *Sender)
{
    dt=6;
    first=true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::DrawGeostatcurve(TObject *Sender) // dynamic part
{
    float SatAzi, SatEle, SatSlantRange;
    int deviation, x, y;
    for (deviation =-90; deviation<90; deviation++)
    {
        GetParametersAndSatellit((float)deviation, &SatAzi, &SatEle, &SatSlantRange);
        x=(int)((float)PaintBox1->Width /360.0*SatAzi);
        y=(int)((float)PaintBox1->Height/90.0*(90.0-SatEle));
        if ((ViewHorizon) &&
            (SatEle>HorizonLevel[(int)SatAzi]) ||
            (!ViewHorizon) )
            PaintBox1->Canvas->Pixels[x][y]=clYellow;
    }
   //PaintBox1->Canvas->Font->Color=clWhite;
   //PaintBox1->Canvas->TextOut(10,50,"Satellit");

}
//---------------------------------------------------------------------------

void __fastcall TForm1::DrawHorizon(TObject *Sender) // dynamic part
{
    int i, x, y0, y1;
    PaintBox1->Canvas->Pen->Color = clGreen;
    for (i =0; i<360; i++)
    {
        x  = (int)((float)PaintBox1->Width /360.0*(float)i);
        y0 = (int)((float)PaintBox1->Height);
        y1 = (int)((float)PaintBox1->Height/90.0*(90.0-HorizonLevel[i]));
        PaintBox1->Canvas->MoveTo(x,y0);
        PaintBox1->Canvas->LineTo(x,y1);
    }
}
//---------------------------------------------------------------------------
void __fastcall TForm1::DrawEvent(float azimut, float elevation) // dynamic part
{
    int x0, y0;
    x0 = (int)((float)PaintBox1->Width /360.0*azimut);
    y0= (int)((float)PaintBox1->Height/90.0*(90.0-elevation));
    PaintBox1->Canvas->Brush->Color = clRed;
    PaintBox1->Canvas->Pen->Color = clRed;
    PaintBox1->Canvas->Brush->Style = bsClear; // bsSolid
    PaintBox1->Canvas->Ellipse(x0-5,y0-5, x0+5, y0+5);   // xl,yo,xr,yu
}
//---------------------------------------------------------------------------
void __fastcall TForm1::HorizontClick(TObject *Sender)
{
    ViewHorizon=!ViewHorizon;
    if (ViewHorizon)
        Horizont->Checked=true;
    else
        Horizont->Checked=false;
    Button5Click(0);
}
//---------------------------------------------------------------------------
void __fastcall TForm1::GeostatClick(TObject *Sender)
{
    GeoCurve=!GeoCurve;
    if (GeoCurve)
        Geostat->Checked=true;
    else
        Geostat->Checked=false;
    Button5Click(0);
}
//---------------------------------------------------------------------------


float GetPart(char *in, int from, int count)
{
    char temp[40];
    int i;
    for (i=from; i<(from+count); i++)
       temp[i-from]=in[i];
    temp[i]='\0';
    return(atof(temp));
}
//---------------------------------------------------------------------------

void __fastcall TForm1::ListBox1Click(TObject *Sender)
{
  char st[40];
  for (int i = 0; i < ListBox1->Items->Count; i++)
  { 
    if (ListBox1->Selected[i])
    {
        sprintf(st,"%s %u",ListBox1->Items->Strings[i],i);
        //Form1->Caption = st;
        DrawEvent(GetPart(st,8,6),GetPart(st,19,6));
    }
  }          
}
//---------------------------------------------------------------------------


void __fastcall TForm1::Button7Click(TObject *Sender)
{
    Graphics::TBitmap* DestBitmap = new Graphics::TBitmap();
    int width=ClientWidth;   // komplette Form1
    int height=ClientHeight;

    DestBitmap->Width=width;
    DestBitmap->Height=height;

    TRect   SrcRect, DesRect;
    SrcRect = Rect(0,0,width, height);
    DesRect = Rect(0,0,width, height);

    DestBitmap->Canvas->CopyRect(SrcRect,Form1->Canvas,DesRect);

    try
    {
      SaveDialog1->Title = "Save Image as bmp-file";
      if (SaveDialog1->Execute())
      {
        if (FileExists(SaveDialog1->FileName))
        DeleteFile(SaveDialog1->FileName);
        DestBitmap->SaveToFile(SaveDialog1->FileName); // Super luft!
        Beep();
      }
    }
    catch(...)
    {
        ShowMessage("Error: Bitmap saving");
    }
    delete DestBitmap;
}
//---------------------------------------------------------------------------


